# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: playwright.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10playwright.proto\"\xdd\x11\n\x07Request\x1a\x07\n\x05\x45mpty\x1a\x16\n\x05Label\x12\r\n\x05label\x18\x01 \x01(\t\x1aT\n\x11ScreenshotOptions\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04mask\x18\x02 \x01(\t\x12\x0f\n\x07options\x18\x03 \x01(\t\x12\x0e\n\x06strict\x18\x04 \x01(\x08\x1a.\n\x0bKeywordCall\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\targuments\x18\x02 \x01(\t\x1a\x18\n\x08\x46ilePath\x12\x0c\n\x04path\x18\x01 \x01(\t\x1a@\n\x0e\x46ileBySelector\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\x12\x0e\n\x06strict\x18\x03 \x01(\x08\x1a\x14\n\x04Json\x12\x0c\n\x04\x62ody\x18\x01 \x01(\t\x1a\x32\n\x12MouseButtonOptions\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\x12\x0c\n\x04json\x18\x02 \x01(\t\x1a/\n\x10KeyboardKeypress\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\x44\n\x14KeyboardInputOptions\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\x12\r\n\x05input\x18\x02 \x01(\t\x12\r\n\x05\x64\x65lay\x18\x03 \x01(\x05\x1a.\n\x07\x42rowser\x12\x0f\n\x07\x62rowser\x18\x01 \x01(\t\x12\x12\n\nrawOptions\x18\x02 \x01(\t\x1a_\n\x07\x43ontext\x12\x12\n\nrawOptions\x18\x01 \x01(\t\x12\x15\n\rhideRfBrowser\x18\x02 \x01(\x08\x12\x16\n\x0e\x64\x65\x66\x61ultTimeout\x18\x03 \x01(\x05\x12\x11\n\ttraceFile\x18\x04 \x01(\t\x1a\x32\n\x0bPermissions\x12\x13\n\x0bpermissions\x18\x01 \x03(\t\x12\x0e\n\x06origin\x18\x02 \x01(\t\x1a*\n\x03Url\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x16\n\x0e\x64\x65\x66\x61ultTimeout\x18\x02 \x01(\x05\x1a:\n\nUrlOptions\x12\x19\n\x03url\x18\x01 \x01(\x0b\x32\x0c.Request.Url\x12\x11\n\twaitUntil\x18\x02 \x01(\t\x1a.\n\x0e\x43onnectBrowser\x12\x0f\n\x07\x62rowser\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x1a:\n\tTextInput\x12\r\n\x05input\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x08\x1a\x45\n\x0f\x45lementProperty\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x10\n\x08property\x18\x02 \x01(\t\x12\x0e\n\x06strict\x18\x03 \x01(\x08\x1aX\n\x08TypeText\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\r\n\x05\x64\x65lay\x18\x03 \x01(\x05\x12\r\n\x05\x63lear\x18\x04 \x01(\x08\x12\x0e\n\x06strict\x18\x05 \x01(\x08\x1aI\n\x08\x46illText\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\x0e\n\x06strict\x18\x03 \x01(\x08\x12\r\n\x05\x66orce\x18\x04 \x01(\x08\x1a-\n\tClearText\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0e\n\x06strict\x18\x02 \x01(\x08\x1a:\n\tPressKeys\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0e\n\x06strict\x18\x02 \x01(\x08\x12\x0b\n\x03key\x18\x03 \x03(\t\x1a\x42\n\x0f\x45lementSelector\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0e\n\x06strict\x18\x02 \x01(\x08\x12\r\n\x05\x66orce\x18\x03 \x01(\x08\x1a\x1a\n\x07Timeout\x12\x0f\n\x07timeout\x18\x01 \x01(\x02\x1a\x16\n\x05Index\x12\r\n\x05index\x18\x01 \x01(\t\x1a,\n\rIdWithTimeout\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07timeout\x18\x02 \x01(\x02\x1a\x1b\n\x08StyleTag\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x1aO\n\x1a\x45lementSelectorWithOptions\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0f\n\x07options\x18\x02 \x01(\t\x12\x0e\n\x06strict\x18\x03 \x01(\x08\x1a~\n\x1b\x45lementSelectorWithDuration\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x10\n\x08\x64uration\x18\x02 \x01(\x05\x12\r\n\x05width\x18\x03 \x01(\t\x12\r\n\x05style\x18\x04 \x01(\t\x12\r\n\x05\x63olor\x18\x05 \x01(\t\x12\x0e\n\x06strict\x18\x06 \x01(\x08\x1aN\n\x15SelectElementSelector\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x13\n\x0bmatcherJson\x18\x02 \x01(\t\x12\x0e\n\x06strict\x18\x03 \x01(\x08\x1a\x42\n\x0eJavascriptCode\x12\x0e\n\x06script\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\x12\x0e\n\x06strict\x18\x03 \x01(\x08\x1a[\n\x16WaitForFunctionOptions\x12\x0e\n\x06script\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\x12\x0f\n\x07options\x18\x03 \x01(\t\x12\x0e\n\x06strict\x18\x04 \x01(\x08\x1a \n\x10PlaywrightObject\x12\x0c\n\x04info\x18\x01 \x01(\t\x1a)\n\x08Viewport\x12\r\n\x05width\x18\x01 \x01(\x05\x12\x0e\n\x06height\x18\x02 \x01(\x05\x1aI\n\x0bHttpRequest\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0e\n\x06method\x18\x02 \x01(\t\x12\x0c\n\x04\x62ody\x18\x03 \x01(\t\x12\x0f\n\x07headers\x18\x04 \x01(\t\x1a\x36\n\x0bHttpCapture\x12\x16\n\x0eurlOrPredicate\x18\x01 \x01(\t\x12\x0f\n\x07timeout\x18\x02 \x01(\x02\x1a\x16\n\x06\x44\x65vice\x12\x0c\n\x04name\x18\x01 \x01(\t\x1a\x37\n\x0b\x41lertAction\x12\x13\n\x0b\x61lertAction\x18\x01 \x01(\t\x12\x13\n\x0bpromptInput\x18\x02 \x01(\t\x1a\x15\n\x04\x42ool\x12\r\n\x05value\x18\x01 \x01(\x08\x1a\x61\n\x0b\x45valuateAll\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0e\n\x06script\x18\x02 \x01(\t\x12\x0b\n\x03\x61rg\x18\x03 \x01(\t\x12\x13\n\x0b\x61llElements\x18\x04 \x01(\x08\x12\x0e\n\x06strict\x18\x05 \x01(\x08\"F\n\x05Types\x1a=\n\x0bSelectEntry\x12\r\n\x05value\x18\x02 \x01(\t\x12\r\n\x05label\x18\x03 \x01(\t\x12\x10\n\x08selected\x18\x04 \x01(\x08\"\x8c\x05\n\x08Response\x1a\x14\n\x05\x45mpty\x12\x0b\n\x03log\x18\x01 \x01(\t\x1a#\n\x06String\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\t\x1a\x62\n\x08Keywords\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x10\n\x08keywords\x18\x02 \x03(\t\x12\x1d\n\x15keywordDocumentations\x18\x03 \x03(\t\x12\x18\n\x10keywordArguments\x18\x04 \x03(\t\x1a!\n\x04\x42ool\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x08\x1a \n\x03Int\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\x05\x1a+\n\x06Select\x12!\n\x05\x65ntry\x18\x01 \x03(\x0b\x32\x12.Types.SelectEntry\x1a!\n\x04Json\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04json\x18\x02 \x01(\t\x1a\x38\n\x19JavascriptExecutionResult\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0e\n\x06result\x18\x02 \x01(\t\x1aY\n\x12NewContextResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0b\n\x03log\x18\x02 \x01(\t\x12\x16\n\x0e\x63ontextOptions\x18\x03 \x01(\t\x12\x12\n\nnewBrowser\x18\x04 \x01(\x08\x1a\x63\n\x0fNewPageResponse\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0c\n\x04\x62ody\x18\x02 \x01(\t\x12\r\n\x05video\x18\x03 \x01(\t\x12\x12\n\nnewBrowser\x18\x04 \x01(\x08\x12\x12\n\nnewContext\x18\x05 \x01(\x08\x1aR\n\x12PageReportResponse\x12\x0b\n\x03log\x18\x01 \x01(\t\x12\x0e\n\x06\x65rrors\x18\x02 \x01(\t\x12\x0f\n\x07\x63onsole\x18\x03 \x01(\t\x12\x0e\n\x06pageId\x18\x04 \x01(\t2\xb6%\n\nPlaywright\x12+\n\tAddCookie\x12\r.Request.Json\x1a\x0f.Response.Empty\x12,\n\nGetCookies\x12\x0e.Request.Empty\x1a\x0e.Response.Json\x12\x33\n\x10\x44\x65leteAllCookies\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12>\n\x0eTakeScreenshot\x12\x1a.Request.ScreenshotOptions\x1a\x10.Response.String\x12%\n\x04GoTo\x12\x0c.Request.Url\x1a\x0f.Response.Empty\x12)\n\x06GoBack\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12,\n\tGoForward\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12,\n\x08GetTitle\x12\x0e.Request.Empty\x1a\x10.Response.String\x12:\n\x0fGetElementCount\x12\x18.Request.ElementSelector\x1a\r.Response.Int\x12.\n\x08TypeText\x12\x11.Request.TypeText\x1a\x0f.Response.Empty\x12.\n\x08\x46illText\x12\x11.Request.FillText\x1a\x0f.Response.Empty\x12\x30\n\tClearText\x12\x12.Request.ClearText\x1a\x0f.Response.Empty\x12<\n\x0eGetDomProperty\x12\x18.Request.ElementProperty\x1a\x10.Response.String\x12\x35\n\x07GetText\x12\x18.Request.ElementSelector\x1a\x10.Response.String\x12\x41\n\x13GetElementAttribute\x12\x18.Request.ElementProperty\x1a\x10.Response.String\x12;\n\x0fGetBoolProperty\x12\x18.Request.ElementProperty\x1a\x0e.Response.Bool\x12\x31\n\x0fGetViewportSize\x12\x0e.Request.Empty\x1a\x0e.Response.Json\x12*\n\x06GetUrl\x12\x0e.Request.Empty\x1a\x10.Response.String\x12\x31\n\rGetPageSource\x12\x0e.Request.Empty\x1a\x10.Response.String\x12,\n\x05Press\x12\x12.Request.PressKeys\x1a\x0f.Response.Empty\x12>\n\x10GetSelectContent\x12\x18.Request.ElementSelector\x1a\x10.Response.Select\x12@\n\x0cSelectOption\x12\x1e.Request.SelectElementSelector\x1a\x10.Response.Select\x12;\n\x0e\x44\x65selectOption\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\x12:\n\rCheckCheckbox\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\x12<\n\x0fUncheckCheckbox\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\x12*\n\x06Health\x12\x0e.Request.Empty\x1a\x10.Response.String\x12\x38\n\nGetElement\x12\x18.Request.ElementSelector\x1a\x10.Response.String\x12\x37\n\x0bGetElements\x12\x18.Request.ElementSelector\x1a\x0e.Response.Json\x12<\n\x10GetElementStates\x12\x18.Request.ElementSelector\x1a\x0e.Response.Json\x12/\n\nSetTimeout\x12\x10.Request.Timeout\x1a\x0f.Response.Empty\x12\x31\n\x0b\x41\x64\x64StyleTag\x12\x11.Request.StyleTag\x1a\x0f.Response.Empty\x12J\n\x11HighlightElements\x12$.Request.ElementSelectorWithDuration\x1a\x0f.Response.Empty\x12(\n\x08\x44ownload\x12\x0c.Request.Url\x1a\x0e.Response.Json\x12=\n\x05\x43lick\x12#.Request.ElementSelectorWithOptions\x1a\x0f.Response.Empty\x12=\n\x05Hover\x12#.Request.ElementSelectorWithOptions\x1a\x0f.Response.Empty\x12\x32\n\x05\x46ocus\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\x12L\n\x14WaitForElementsState\x12#.Request.ElementSelectorWithOptions\x1a\x0f.Response.Empty\x12\x42\n\x0fWaitForFunction\x12\x1f.Request.WaitForFunctionOptions\x1a\x0e.Response.Json\x12Q\n\x11\x45xecuteJavascript\x12\x17.Request.JavascriptCode\x1a#.Response.JavascriptExecutionResult\x12O\n\x12\x45valuateJavascript\x12\x14.Request.EvaluateAll\x1a#.Response.JavascriptExecutionResult\x12\x45\n\x0eRecordSelector\x12\x0e.Request.Label\x1a#.Response.JavascriptExecutionResult\x12\x43\n\x0cGetPageState\x12\x0e.Request.Empty\x1a#.Response.JavascriptExecutionResult\x12\x35\n\x0fSetViewportSize\x12\x11.Request.Viewport\x1a\x0f.Response.Empty\x12\x34\n\x08GetStyle\x12\x18.Request.ElementSelector\x1a\x0e.Response.Json\x12:\n\x0eGetBoundingBox\x12\x18.Request.ElementSelector\x1a\x0e.Response.Json\x12\x33\n\x0bHttpRequest\x12\x14.Request.HttpRequest\x1a\x0e.Response.Json\x12\x38\n\x0eWaitForRequest\x12\x14.Request.HttpCapture\x1a\x10.Response.String\x12\x37\n\x0fWaitForResponse\x12\x14.Request.HttpCapture\x1a\x0e.Response.Json\x12\x34\n\x0fWaitForDownload\x12\x11.Request.FilePath\x1a\x0e.Response.Json\x12;\n\x16WaitUntilNetworkIsIdle\x12\x10.Request.Timeout\x1a\x0f.Response.Empty\x12\x39\n\x11WaitForNavigation\x12\x13.Request.UrlOptions\x1a\x0f.Response.Empty\x12\x30\n\x0eSetGeolocation\x12\r.Request.Json\x1a\x0f.Response.Empty\x12,\n\tGetDevice\x12\x0f.Request.Device\x1a\x0e.Response.Json\x12,\n\nGetDevices\x12\x0e.Request.Empty\x1a\x0e.Response.Json\x12\x34\n\x0bHandleAlert\x12\x14.Request.AlertAction\x1a\x0f.Response.Empty\x12\x36\n\x0cWaitForAlert\x12\x14.Request.AlertAction\x1a\x10.Response.String\x12;\n\x0bMouseButton\x12\x1b.Request.MouseButtonOptions\x1a\x0f.Response.Empty\x12+\n\tMouseMove\x12\r.Request.Json\x1a\x0f.Response.Empty\x12\x39\n\x0bKeyboardKey\x12\x19.Request.KeyboardKeypress\x1a\x0f.Response.Empty\x12?\n\rKeyboardInput\x12\x1d.Request.KeyboardInputOptions\x1a\x0f.Response.Empty\x12;\n\x10GetTableRowIndex\x12\x18.Request.ElementSelector\x1a\r.Response.Int\x12<\n\x11GetTableCellIndex\x12\x18.Request.ElementSelector\x1a\r.Response.Int\x12\x30\n\nUploadFile\x12\x11.Request.FilePath\x1a\x0f.Response.Empty\x12@\n\x14UploadFileBySelector\x12\x17.Request.FileBySelector\x1a\x0f.Response.Empty\x12<\n\x0fScrollToElement\x12\x18.Request.ElementSelector\x1a\x0f.Response.Empty\x12,\n\nSetOffline\x12\r.Request.Bool\x1a\x0f.Response.Empty\x12)\n\x06Reload\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12\x36\n\nSwitchPage\x12\x16.Request.IdWithTimeout\x1a\x10.Response.String\x12\x31\n\rSwitchContext\x12\x0e.Request.Index\x1a\x10.Response.String\x12\x31\n\rSwitchBrowser\x12\x0e.Request.Index\x1a\x10.Response.String\x12\x32\n\x07NewPage\x12\x0c.Request.Url\x1a\x19.Response.NewPageResponse\x12<\n\nNewContext\x12\x10.Request.Context\x1a\x1c.Response.NewContextResponse\x12\x30\n\nNewBrowser\x12\x10.Request.Browser\x1a\x10.Response.String\x12=\n\x10\x43onnectToBrowser\x12\x17.Request.ConnectBrowser\x1a\x10.Response.String\x12\x30\n\x0c\x43loseBrowser\x12\x0e.Request.Empty\x1a\x10.Response.String\x12\x33\n\x10\x43loseAllBrowsers\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12/\n\x0c\x43loseContext\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12\x39\n\tClosePage\x12\x0e.Request.Empty\x1a\x1c.Response.PageReportResponse\x12\x33\n\x11GetBrowserCatalog\x12\x0e.Request.Empty\x1a\x0e.Response.Json\x12\x36\n\x10SaveStorageState\x12\x11.Request.FilePath\x1a\x0f.Response.Empty\x12\x39\n\x10GrantPermissions\x12\x14.Request.Permissions\x1a\x0f.Response.Empty\x12\x33\n\x10\x43learPermissions\x12\x0e.Request.Empty\x1a\x0f.Response.Empty\x12<\n\x13InitializeExtension\x12\x11.Request.FilePath\x1a\x12.Response.Keywords\x12>\n\x14\x43\x61llExtensionKeyword\x12\x14.Request.KeywordCall\x1a\x0e.Response.Json0\x01\x12-\n\tSetPeerId\x12\x0e.Request.Index\x1a\x10.Response.Stringb\x06proto3')



_REQUEST = DESCRIPTOR.message_types_by_name['Request']
_REQUEST_EMPTY = _REQUEST.nested_types_by_name['Empty']
_REQUEST_LABEL = _REQUEST.nested_types_by_name['Label']
_REQUEST_SCREENSHOTOPTIONS = _REQUEST.nested_types_by_name['ScreenshotOptions']
_REQUEST_KEYWORDCALL = _REQUEST.nested_types_by_name['KeywordCall']
_REQUEST_FILEPATH = _REQUEST.nested_types_by_name['FilePath']
_REQUEST_FILEBYSELECTOR = _REQUEST.nested_types_by_name['FileBySelector']
_REQUEST_JSON = _REQUEST.nested_types_by_name['Json']
_REQUEST_MOUSEBUTTONOPTIONS = _REQUEST.nested_types_by_name['MouseButtonOptions']
_REQUEST_KEYBOARDKEYPRESS = _REQUEST.nested_types_by_name['KeyboardKeypress']
_REQUEST_KEYBOARDINPUTOPTIONS = _REQUEST.nested_types_by_name['KeyboardInputOptions']
_REQUEST_BROWSER = _REQUEST.nested_types_by_name['Browser']
_REQUEST_CONTEXT = _REQUEST.nested_types_by_name['Context']
_REQUEST_PERMISSIONS = _REQUEST.nested_types_by_name['Permissions']
_REQUEST_URL = _REQUEST.nested_types_by_name['Url']
_REQUEST_URLOPTIONS = _REQUEST.nested_types_by_name['UrlOptions']
_REQUEST_CONNECTBROWSER = _REQUEST.nested_types_by_name['ConnectBrowser']
_REQUEST_TEXTINPUT = _REQUEST.nested_types_by_name['TextInput']
_REQUEST_ELEMENTPROPERTY = _REQUEST.nested_types_by_name['ElementProperty']
_REQUEST_TYPETEXT = _REQUEST.nested_types_by_name['TypeText']
_REQUEST_FILLTEXT = _REQUEST.nested_types_by_name['FillText']
_REQUEST_CLEARTEXT = _REQUEST.nested_types_by_name['ClearText']
_REQUEST_PRESSKEYS = _REQUEST.nested_types_by_name['PressKeys']
_REQUEST_ELEMENTSELECTOR = _REQUEST.nested_types_by_name['ElementSelector']
_REQUEST_TIMEOUT = _REQUEST.nested_types_by_name['Timeout']
_REQUEST_INDEX = _REQUEST.nested_types_by_name['Index']
_REQUEST_IDWITHTIMEOUT = _REQUEST.nested_types_by_name['IdWithTimeout']
_REQUEST_STYLETAG = _REQUEST.nested_types_by_name['StyleTag']
_REQUEST_ELEMENTSELECTORWITHOPTIONS = _REQUEST.nested_types_by_name['ElementSelectorWithOptions']
_REQUEST_ELEMENTSELECTORWITHDURATION = _REQUEST.nested_types_by_name['ElementSelectorWithDuration']
_REQUEST_SELECTELEMENTSELECTOR = _REQUEST.nested_types_by_name['SelectElementSelector']
_REQUEST_JAVASCRIPTCODE = _REQUEST.nested_types_by_name['JavascriptCode']
_REQUEST_WAITFORFUNCTIONOPTIONS = _REQUEST.nested_types_by_name['WaitForFunctionOptions']
_REQUEST_PLAYWRIGHTOBJECT = _REQUEST.nested_types_by_name['PlaywrightObject']
_REQUEST_VIEWPORT = _REQUEST.nested_types_by_name['Viewport']
_REQUEST_HTTPREQUEST = _REQUEST.nested_types_by_name['HttpRequest']
_REQUEST_HTTPCAPTURE = _REQUEST.nested_types_by_name['HttpCapture']
_REQUEST_DEVICE = _REQUEST.nested_types_by_name['Device']
_REQUEST_ALERTACTION = _REQUEST.nested_types_by_name['AlertAction']
_REQUEST_BOOL = _REQUEST.nested_types_by_name['Bool']
_REQUEST_EVALUATEALL = _REQUEST.nested_types_by_name['EvaluateAll']
_TYPES = DESCRIPTOR.message_types_by_name['Types']
_TYPES_SELECTENTRY = _TYPES.nested_types_by_name['SelectEntry']
_RESPONSE = DESCRIPTOR.message_types_by_name['Response']
_RESPONSE_EMPTY = _RESPONSE.nested_types_by_name['Empty']
_RESPONSE_STRING = _RESPONSE.nested_types_by_name['String']
_RESPONSE_KEYWORDS = _RESPONSE.nested_types_by_name['Keywords']
_RESPONSE_BOOL = _RESPONSE.nested_types_by_name['Bool']
_RESPONSE_INT = _RESPONSE.nested_types_by_name['Int']
_RESPONSE_SELECT = _RESPONSE.nested_types_by_name['Select']
_RESPONSE_JSON = _RESPONSE.nested_types_by_name['Json']
_RESPONSE_JAVASCRIPTEXECUTIONRESULT = _RESPONSE.nested_types_by_name['JavascriptExecutionResult']
_RESPONSE_NEWCONTEXTRESPONSE = _RESPONSE.nested_types_by_name['NewContextResponse']
_RESPONSE_NEWPAGERESPONSE = _RESPONSE.nested_types_by_name['NewPageResponse']
_RESPONSE_PAGEREPORTRESPONSE = _RESPONSE.nested_types_by_name['PageReportResponse']
Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), {

  'Empty' : _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_EMPTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Empty)
    })
  ,

  'Label' : _reflection.GeneratedProtocolMessageType('Label', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_LABEL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Label)
    })
  ,

  'ScreenshotOptions' : _reflection.GeneratedProtocolMessageType('ScreenshotOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_SCREENSHOTOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ScreenshotOptions)
    })
  ,

  'KeywordCall' : _reflection.GeneratedProtocolMessageType('KeywordCall', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_KEYWORDCALL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.KeywordCall)
    })
  ,

  'FilePath' : _reflection.GeneratedProtocolMessageType('FilePath', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_FILEPATH,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.FilePath)
    })
  ,

  'FileBySelector' : _reflection.GeneratedProtocolMessageType('FileBySelector', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_FILEBYSELECTOR,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.FileBySelector)
    })
  ,

  'Json' : _reflection.GeneratedProtocolMessageType('Json', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_JSON,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Json)
    })
  ,

  'MouseButtonOptions' : _reflection.GeneratedProtocolMessageType('MouseButtonOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_MOUSEBUTTONOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.MouseButtonOptions)
    })
  ,

  'KeyboardKeypress' : _reflection.GeneratedProtocolMessageType('KeyboardKeypress', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_KEYBOARDKEYPRESS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.KeyboardKeypress)
    })
  ,

  'KeyboardInputOptions' : _reflection.GeneratedProtocolMessageType('KeyboardInputOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_KEYBOARDINPUTOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.KeyboardInputOptions)
    })
  ,

  'Browser' : _reflection.GeneratedProtocolMessageType('Browser', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_BROWSER,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Browser)
    })
  ,

  'Context' : _reflection.GeneratedProtocolMessageType('Context', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_CONTEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Context)
    })
  ,

  'Permissions' : _reflection.GeneratedProtocolMessageType('Permissions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_PERMISSIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Permissions)
    })
  ,

  'Url' : _reflection.GeneratedProtocolMessageType('Url', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_URL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Url)
    })
  ,

  'UrlOptions' : _reflection.GeneratedProtocolMessageType('UrlOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_URLOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.UrlOptions)
    })
  ,

  'ConnectBrowser' : _reflection.GeneratedProtocolMessageType('ConnectBrowser', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_CONNECTBROWSER,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ConnectBrowser)
    })
  ,

  'TextInput' : _reflection.GeneratedProtocolMessageType('TextInput', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TEXTINPUT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.TextInput)
    })
  ,

  'ElementProperty' : _reflection.GeneratedProtocolMessageType('ElementProperty', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTPROPERTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementProperty)
    })
  ,

  'TypeText' : _reflection.GeneratedProtocolMessageType('TypeText', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TYPETEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.TypeText)
    })
  ,

  'FillText' : _reflection.GeneratedProtocolMessageType('FillText', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_FILLTEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.FillText)
    })
  ,

  'ClearText' : _reflection.GeneratedProtocolMessageType('ClearText', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_CLEARTEXT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ClearText)
    })
  ,

  'PressKeys' : _reflection.GeneratedProtocolMessageType('PressKeys', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_PRESSKEYS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.PressKeys)
    })
  ,

  'ElementSelector' : _reflection.GeneratedProtocolMessageType('ElementSelector', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTSELECTOR,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementSelector)
    })
  ,

  'Timeout' : _reflection.GeneratedProtocolMessageType('Timeout', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_TIMEOUT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Timeout)
    })
  ,

  'Index' : _reflection.GeneratedProtocolMessageType('Index', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_INDEX,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Index)
    })
  ,

  'IdWithTimeout' : _reflection.GeneratedProtocolMessageType('IdWithTimeout', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_IDWITHTIMEOUT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.IdWithTimeout)
    })
  ,

  'StyleTag' : _reflection.GeneratedProtocolMessageType('StyleTag', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_STYLETAG,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.StyleTag)
    })
  ,

  'ElementSelectorWithOptions' : _reflection.GeneratedProtocolMessageType('ElementSelectorWithOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTSELECTORWITHOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementSelectorWithOptions)
    })
  ,

  'ElementSelectorWithDuration' : _reflection.GeneratedProtocolMessageType('ElementSelectorWithDuration', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ELEMENTSELECTORWITHDURATION,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.ElementSelectorWithDuration)
    })
  ,

  'SelectElementSelector' : _reflection.GeneratedProtocolMessageType('SelectElementSelector', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_SELECTELEMENTSELECTOR,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.SelectElementSelector)
    })
  ,

  'JavascriptCode' : _reflection.GeneratedProtocolMessageType('JavascriptCode', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_JAVASCRIPTCODE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.JavascriptCode)
    })
  ,

  'WaitForFunctionOptions' : _reflection.GeneratedProtocolMessageType('WaitForFunctionOptions', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_WAITFORFUNCTIONOPTIONS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.WaitForFunctionOptions)
    })
  ,

  'PlaywrightObject' : _reflection.GeneratedProtocolMessageType('PlaywrightObject', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_PLAYWRIGHTOBJECT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.PlaywrightObject)
    })
  ,

  'Viewport' : _reflection.GeneratedProtocolMessageType('Viewport', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_VIEWPORT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Viewport)
    })
  ,

  'HttpRequest' : _reflection.GeneratedProtocolMessageType('HttpRequest', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_HTTPREQUEST,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.HttpRequest)
    })
  ,

  'HttpCapture' : _reflection.GeneratedProtocolMessageType('HttpCapture', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_HTTPCAPTURE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.HttpCapture)
    })
  ,

  'Device' : _reflection.GeneratedProtocolMessageType('Device', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_DEVICE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Device)
    })
  ,

  'AlertAction' : _reflection.GeneratedProtocolMessageType('AlertAction', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_ALERTACTION,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.AlertAction)
    })
  ,

  'Bool' : _reflection.GeneratedProtocolMessageType('Bool', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_BOOL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.Bool)
    })
  ,

  'EvaluateAll' : _reflection.GeneratedProtocolMessageType('EvaluateAll', (_message.Message,), {
    'DESCRIPTOR' : _REQUEST_EVALUATEALL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Request.EvaluateAll)
    })
  ,
  'DESCRIPTOR' : _REQUEST,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Request)
  })
_sym_db.RegisterMessage(Request)
_sym_db.RegisterMessage(Request.Empty)
_sym_db.RegisterMessage(Request.Label)
_sym_db.RegisterMessage(Request.ScreenshotOptions)
_sym_db.RegisterMessage(Request.KeywordCall)
_sym_db.RegisterMessage(Request.FilePath)
_sym_db.RegisterMessage(Request.FileBySelector)
_sym_db.RegisterMessage(Request.Json)
_sym_db.RegisterMessage(Request.MouseButtonOptions)
_sym_db.RegisterMessage(Request.KeyboardKeypress)
_sym_db.RegisterMessage(Request.KeyboardInputOptions)
_sym_db.RegisterMessage(Request.Browser)
_sym_db.RegisterMessage(Request.Context)
_sym_db.RegisterMessage(Request.Permissions)
_sym_db.RegisterMessage(Request.Url)
_sym_db.RegisterMessage(Request.UrlOptions)
_sym_db.RegisterMessage(Request.ConnectBrowser)
_sym_db.RegisterMessage(Request.TextInput)
_sym_db.RegisterMessage(Request.ElementProperty)
_sym_db.RegisterMessage(Request.TypeText)
_sym_db.RegisterMessage(Request.FillText)
_sym_db.RegisterMessage(Request.ClearText)
_sym_db.RegisterMessage(Request.PressKeys)
_sym_db.RegisterMessage(Request.ElementSelector)
_sym_db.RegisterMessage(Request.Timeout)
_sym_db.RegisterMessage(Request.Index)
_sym_db.RegisterMessage(Request.IdWithTimeout)
_sym_db.RegisterMessage(Request.StyleTag)
_sym_db.RegisterMessage(Request.ElementSelectorWithOptions)
_sym_db.RegisterMessage(Request.ElementSelectorWithDuration)
_sym_db.RegisterMessage(Request.SelectElementSelector)
_sym_db.RegisterMessage(Request.JavascriptCode)
_sym_db.RegisterMessage(Request.WaitForFunctionOptions)
_sym_db.RegisterMessage(Request.PlaywrightObject)
_sym_db.RegisterMessage(Request.Viewport)
_sym_db.RegisterMessage(Request.HttpRequest)
_sym_db.RegisterMessage(Request.HttpCapture)
_sym_db.RegisterMessage(Request.Device)
_sym_db.RegisterMessage(Request.AlertAction)
_sym_db.RegisterMessage(Request.Bool)
_sym_db.RegisterMessage(Request.EvaluateAll)

Types = _reflection.GeneratedProtocolMessageType('Types', (_message.Message,), {

  'SelectEntry' : _reflection.GeneratedProtocolMessageType('SelectEntry', (_message.Message,), {
    'DESCRIPTOR' : _TYPES_SELECTENTRY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Types.SelectEntry)
    })
  ,
  'DESCRIPTOR' : _TYPES,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Types)
  })
_sym_db.RegisterMessage(Types)
_sym_db.RegisterMessage(Types.SelectEntry)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {

  'Empty' : _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_EMPTY,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Empty)
    })
  ,

  'String' : _reflection.GeneratedProtocolMessageType('String', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_STRING,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.String)
    })
  ,

  'Keywords' : _reflection.GeneratedProtocolMessageType('Keywords', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_KEYWORDS,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Keywords)
    })
  ,

  'Bool' : _reflection.GeneratedProtocolMessageType('Bool', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_BOOL,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Bool)
    })
  ,

  'Int' : _reflection.GeneratedProtocolMessageType('Int', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_INT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Int)
    })
  ,

  'Select' : _reflection.GeneratedProtocolMessageType('Select', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_SELECT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Select)
    })
  ,

  'Json' : _reflection.GeneratedProtocolMessageType('Json', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_JSON,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.Json)
    })
  ,

  'JavascriptExecutionResult' : _reflection.GeneratedProtocolMessageType('JavascriptExecutionResult', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_JAVASCRIPTEXECUTIONRESULT,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.JavascriptExecutionResult)
    })
  ,

  'NewContextResponse' : _reflection.GeneratedProtocolMessageType('NewContextResponse', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_NEWCONTEXTRESPONSE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.NewContextResponse)
    })
  ,

  'NewPageResponse' : _reflection.GeneratedProtocolMessageType('NewPageResponse', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_NEWPAGERESPONSE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.NewPageResponse)
    })
  ,

  'PageReportResponse' : _reflection.GeneratedProtocolMessageType('PageReportResponse', (_message.Message,), {
    'DESCRIPTOR' : _RESPONSE_PAGEREPORTRESPONSE,
    '__module__' : 'playwright_pb2'
    # @@protoc_insertion_point(class_scope:Response.PageReportResponse)
    })
  ,
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'playwright_pb2'
  # @@protoc_insertion_point(class_scope:Response)
  })
_sym_db.RegisterMessage(Response)
_sym_db.RegisterMessage(Response.Empty)
_sym_db.RegisterMessage(Response.String)
_sym_db.RegisterMessage(Response.Keywords)
_sym_db.RegisterMessage(Response.Bool)
_sym_db.RegisterMessage(Response.Int)
_sym_db.RegisterMessage(Response.Select)
_sym_db.RegisterMessage(Response.Json)
_sym_db.RegisterMessage(Response.JavascriptExecutionResult)
_sym_db.RegisterMessage(Response.NewContextResponse)
_sym_db.RegisterMessage(Response.NewPageResponse)
_sym_db.RegisterMessage(Response.PageReportResponse)

_PLAYWRIGHT = DESCRIPTOR.services_by_name['Playwright']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _REQUEST._serialized_start=21
  _REQUEST._serialized_end=2290
  _REQUEST_EMPTY._serialized_start=32
  _REQUEST_EMPTY._serialized_end=39
  _REQUEST_LABEL._serialized_start=41
  _REQUEST_LABEL._serialized_end=63
  _REQUEST_SCREENSHOTOPTIONS._serialized_start=65
  _REQUEST_SCREENSHOTOPTIONS._serialized_end=149
  _REQUEST_KEYWORDCALL._serialized_start=151
  _REQUEST_KEYWORDCALL._serialized_end=197
  _REQUEST_FILEPATH._serialized_start=199
  _REQUEST_FILEPATH._serialized_end=223
  _REQUEST_FILEBYSELECTOR._serialized_start=225
  _REQUEST_FILEBYSELECTOR._serialized_end=289
  _REQUEST_JSON._serialized_start=291
  _REQUEST_JSON._serialized_end=311
  _REQUEST_MOUSEBUTTONOPTIONS._serialized_start=313
  _REQUEST_MOUSEBUTTONOPTIONS._serialized_end=363
  _REQUEST_KEYBOARDKEYPRESS._serialized_start=365
  _REQUEST_KEYBOARDKEYPRESS._serialized_end=412
  _REQUEST_KEYBOARDINPUTOPTIONS._serialized_start=414
  _REQUEST_KEYBOARDINPUTOPTIONS._serialized_end=482
  _REQUEST_BROWSER._serialized_start=484
  _REQUEST_BROWSER._serialized_end=530
  _REQUEST_CONTEXT._serialized_start=532
  _REQUEST_CONTEXT._serialized_end=627
  _REQUEST_PERMISSIONS._serialized_start=629
  _REQUEST_PERMISSIONS._serialized_end=679
  _REQUEST_URL._serialized_start=681
  _REQUEST_URL._serialized_end=723
  _REQUEST_URLOPTIONS._serialized_start=725
  _REQUEST_URLOPTIONS._serialized_end=783
  _REQUEST_CONNECTBROWSER._serialized_start=785
  _REQUEST_CONNECTBROWSER._serialized_end=831
  _REQUEST_TEXTINPUT._serialized_start=833
  _REQUEST_TEXTINPUT._serialized_end=891
  _REQUEST_ELEMENTPROPERTY._serialized_start=893
  _REQUEST_ELEMENTPROPERTY._serialized_end=962
  _REQUEST_TYPETEXT._serialized_start=964
  _REQUEST_TYPETEXT._serialized_end=1052
  _REQUEST_FILLTEXT._serialized_start=1054
  _REQUEST_FILLTEXT._serialized_end=1127
  _REQUEST_CLEARTEXT._serialized_start=1129
  _REQUEST_CLEARTEXT._serialized_end=1174
  _REQUEST_PRESSKEYS._serialized_start=1176
  _REQUEST_PRESSKEYS._serialized_end=1234
  _REQUEST_ELEMENTSELECTOR._serialized_start=1236
  _REQUEST_ELEMENTSELECTOR._serialized_end=1302
  _REQUEST_TIMEOUT._serialized_start=1304
  _REQUEST_TIMEOUT._serialized_end=1330
  _REQUEST_INDEX._serialized_start=1332
  _REQUEST_INDEX._serialized_end=1354
  _REQUEST_IDWITHTIMEOUT._serialized_start=1356
  _REQUEST_IDWITHTIMEOUT._serialized_end=1400
  _REQUEST_STYLETAG._serialized_start=1402
  _REQUEST_STYLETAG._serialized_end=1429
  _REQUEST_ELEMENTSELECTORWITHOPTIONS._serialized_start=1431
  _REQUEST_ELEMENTSELECTORWITHOPTIONS._serialized_end=1510
  _REQUEST_ELEMENTSELECTORWITHDURATION._serialized_start=1512
  _REQUEST_ELEMENTSELECTORWITHDURATION._serialized_end=1638
  _REQUEST_SELECTELEMENTSELECTOR._serialized_start=1640
  _REQUEST_SELECTELEMENTSELECTOR._serialized_end=1718
  _REQUEST_JAVASCRIPTCODE._serialized_start=1720
  _REQUEST_JAVASCRIPTCODE._serialized_end=1786
  _REQUEST_WAITFORFUNCTIONOPTIONS._serialized_start=1788
  _REQUEST_WAITFORFUNCTIONOPTIONS._serialized_end=1879
  _REQUEST_PLAYWRIGHTOBJECT._serialized_start=1881
  _REQUEST_PLAYWRIGHTOBJECT._serialized_end=1913
  _REQUEST_VIEWPORT._serialized_start=1915
  _REQUEST_VIEWPORT._serialized_end=1956
  _REQUEST_HTTPREQUEST._serialized_start=1958
  _REQUEST_HTTPREQUEST._serialized_end=2031
  _REQUEST_HTTPCAPTURE._serialized_start=2033
  _REQUEST_HTTPCAPTURE._serialized_end=2087
  _REQUEST_DEVICE._serialized_start=2089
  _REQUEST_DEVICE._serialized_end=2111
  _REQUEST_ALERTACTION._serialized_start=2113
  _REQUEST_ALERTACTION._serialized_end=2168
  _REQUEST_BOOL._serialized_start=2170
  _REQUEST_BOOL._serialized_end=2191
  _REQUEST_EVALUATEALL._serialized_start=2193
  _REQUEST_EVALUATEALL._serialized_end=2290
  _TYPES._serialized_start=2292
  _TYPES._serialized_end=2362
  _TYPES_SELECTENTRY._serialized_start=2301
  _TYPES_SELECTENTRY._serialized_end=2362
  _RESPONSE._serialized_start=2365
  _RESPONSE._serialized_end=3017
  _RESPONSE_EMPTY._serialized_start=2377
  _RESPONSE_EMPTY._serialized_end=2397
  _RESPONSE_STRING._serialized_start=2399
  _RESPONSE_STRING._serialized_end=2434
  _RESPONSE_KEYWORDS._serialized_start=2436
  _RESPONSE_KEYWORDS._serialized_end=2534
  _RESPONSE_BOOL._serialized_start=2536
  _RESPONSE_BOOL._serialized_end=2569
  _RESPONSE_INT._serialized_start=2571
  _RESPONSE_INT._serialized_end=2603
  _RESPONSE_SELECT._serialized_start=2605
  _RESPONSE_SELECT._serialized_end=2648
  _RESPONSE_JSON._serialized_start=2650
  _RESPONSE_JSON._serialized_end=2683
  _RESPONSE_JAVASCRIPTEXECUTIONRESULT._serialized_start=2685
  _RESPONSE_JAVASCRIPTEXECUTIONRESULT._serialized_end=2741
  _RESPONSE_NEWCONTEXTRESPONSE._serialized_start=2743
  _RESPONSE_NEWCONTEXTRESPONSE._serialized_end=2832
  _RESPONSE_NEWPAGERESPONSE._serialized_start=2834
  _RESPONSE_NEWPAGERESPONSE._serialized_end=2933
  _RESPONSE_PAGEREPORTRESPONSE._serialized_start=2935
  _RESPONSE_PAGEREPORTRESPONSE._serialized_end=3017
  _PLAYWRIGHT._serialized_start=3020
  _PLAYWRIGHT._serialized_end=7810
# @@protoc_insertion_point(module_scope)
